# Game Play Mode: CTF
# Mod: CE Legends (http://www.celegends.com)
#
# Authors: SuperB, Rexman
#

g_flag_playerInx = [-1,-1,-1]
g_flag_otherTeam = [0,2,1]
g_flag_names = ["noTeamFlag", "MyRedFlag", "MyBlueFlag"]
g_team_names = ["neutral","Red Team","Blue Team"]
g_carryFlagNames = ["GrayFlagPCO","BlueCarryFlag","RedCarryFlag"]
g_soldier_types =["ch_light_soldier","ch_heavy_soldier","mec_light_soldier","mec_heavy_soldier","us_light_soldier","us_heavy_soldier","ce_blue_soldier","ce_red_soldier","ce_brown_soldier"]
g_interval = 1 # timer spacing in seconds
g_miniMapFlagNames = ["GrayFlagPCO","BlueMiniMapFlag","RedMiniMapFlag"] #flag display implementation
g_timeDroppedFlag = [-1,-1,-1]#dropped flag implementation
g_allowOldStyleFlags = 0 #flag display toggle
g_allowdroppedflags = 1 #dropped flag toggle
g_timeForDroppedFlagsToReturn = 30 #dropped flag return time

CarryFlagCP_Object = None

TAKEOVERTYPE_CAPTURE = 1
TAKEOVERTYPE_NEUTRALIZE = 2

SCORE_CAPTURE = 2
SCORE_NEUTRALIZE = 2
SCORE_CAPTUREASSIST = 1
SCORE_NEUTRALIZEASSIST = 1
SCORE_DEFEND = 1

Top = 0
Middle = 1
Bottom = 2

import host
import bf2
import math
import bf2.Timer
from game.scoringCommon import addScore, RPL
from bf2 import g_debug

g_controlPoints = [] # cache, as this map won't change


# DEBUG PRINT
def prnt(myvariable):
        rcon_command = 'game.sayall "' + "C3 P " +str(myvariable) + '"' 
        host.rcon_invoke(rcon_command)


def init():
	# events hook
	host.registerGameStatusHandler(onGameStatusChanged)
	if host.sgl_getIsAIGame() == 1:
		host.sh_setEnableCommander(1)
	else:
		host.sh_setEnableCommander(1)
		
	host.registerHandler('TimeLimitReached', onTimeLimitReached, 1)

	# This is our looping function call to update visuals
	interval = g_interval
	timer = bf2.Timer(onTimer, interval, 1)
	timer.setRecurring(interval)

	if g_debug: print "gpm_ctf.py initialized"
		
		
		
def deinit():
	bf2.triggerManager.destroyAllTriggers()
	
	host.unregisterGameStatusHandler(onTimer)
        host.timer_destroy(onTimer)
         
	global g_controlPoints
	g_controlPoints = []
	host.unregisterGameStatusHandler(onGameStatusChanged)
	if g_debug: print "gpm_ctf.py uninitialized"



def onGameStatusChanged(status):

	global g_controlPoints

	
	if status == bf2.GameStatus.Playing: 

		# add control point triggers
		g_controlPoints = bf2.objectManager.getObjectsOfType('dice.hfe.world.ObjectTemplate.ControlPoint')

                
		for obj in g_controlPoints:

                    if not obj.templateName in g_carryFlagNames: # carry flags don't have triggers
			radius = float(obj.getTemplateProperty('radius'))
			isHemi = int(obj.cp_getParam('isHemisphere'))
			if isHemi != 0:
				id = bf2.triggerManager.createHemiSphericalTrigger(obj, onCPTrigger, '<<PCO>>', radius, (1, 2, 3))
			else:
				id = bf2.triggerManager.createRadiusTrigger(obj, onCPTrigger, '<<PCO>>', radius, (1, 2, 3))			
			obj.triggerId = id
			obj.lastAttackingTeam = 0
			if obj.cp_getParam('team') > 0:
				obj.flagPosition = Top
			else:
				obj.flagPosition = Bottom

		host.registerHandler('ControlPointChangedOwner', onCPStatusChange)
                resetctf()
		# setup ticket system
		ticketsTeam1 = calcStartTickets(bf2.gameLogic.getDefaultTickets(1))
		ticketsTeam2 = calcStartTickets(bf2.gameLogic.getDefaultTickets(2))
                zeroticketsystem = 1000
		# starting ticket count
		bf2.gameLogic.setTickets(1, zeroticketsystem)
		bf2.gameLogic.setTickets(2, zeroticketsystem)

		bf2.gameLogic.setTicketState(1, 0)
		bf2.gameLogic.setTicketState(2, 0)

		bf2.gameLogic.setTicketLimit(1, 1, 0) #low limit target
		bf2.gameLogic.setTicketLimit(2, 1, 0)
		bf2.gameLogic.setTicketLimit(1, 2, -1) #heartbeat sound
		bf2.gameLogic.setTicketLimit(2, 2, -1)
		bf2.gameLogic.setTicketLimit(1, 3, int(zeroticketsystem+int(ticketsTeam1)*0.9)) #low on reinforcements
		bf2.gameLogic.setTicketLimit(2, 3, int(zeroticketsystem+int(ticketsTeam2)*0.9))
		bf2.gameLogic.setTicketLimit(1, 4, int(zeroticketsystem+int(ticketsTeam1)*0.95)) #weak in key positions
		bf2.gameLogic.setTicketLimit(2, 4, int(zeroticketsystem+int(ticketsTeam2)*0.95))
		
		host.registerHandler('TicketLimitReached', onTicketLimitReached)

		# player events
		host.registerHandler('PlayerDeath', onPlayerDeathCQ)
		host.registerHandler('PlayerKilled', onPlayerKilledCQ)
		host.registerHandler('PlayerRevived', onPlayerRevived)
		host.registerHandler('PlayerSpawn', onPlayerSpawn)
		host.registerHandler('EnterVehicle', onEnterVehicle)
		host.registerHandler('ExitVehicle', onExitVehicle)
		host.registerHandler('ChatMessage', onChatMessage)
                host.registerHandler('PlayerDisconnect', onPlayerDisconnect)
		print "CTF gamemode initialized. 0.4"
	else:
		bf2.triggerManager.destroyAllTriggers()
		g_controlPoints = []


def onChatMessage(playerId, text, channel, flags): 
   global first 
   global second 
   if text.find('Dogzilla') == 0: 
      print host.rcon_invoke('game.sayAll "Dogzilla get kisses"') 
   elif text.find('Snazz') == 0: 
      print host.rcon_invoke('game.sayAll "Snazz get kisses"') 


def calcStartTickets(mapDefaultTickets):
	return int(mapDefaultTickets * (bf2.serverSettings.getTicketRatio() / 100.0) * 0.1)
	
	
	
def onTimeLimitReached(value):
	team1tickets = bf2.gameLogic.getTickets(1)
	team2tickets = bf2.gameLogic.getTickets(2)
	
	winner = 0
	victoryType = 0
	if team1tickets > team2tickets:
		winner = 1
		victoryType = 3
	elif team2tickets > team1tickets:
		winner = 2
		victoryType = 3
	

	host.sgl_endGame(winner, victoryType)



# called when tickets reach a predetermined limit (negative value means that the tickets have become less than the limit)
def onTicketLimitReached(team, limitId):
        global playersTeamTickets_end
        global otherTeamTickets_end
        global playersTeam_end
        global otherTeam_end
        bf2.gameLogic.setTickets( playersTeam_end , playersTeamTickets_end)
        bf2.gameLogic.setTickets( otherTeam_end , otherTeamTickets_end)
	if (limitId == -1):
		if (team == 1):
			winner = 2
		
		elif (team == 2):
			winner = 1
		
		bf2.gameLogic.setTicketState(1, 0)
		bf2.gameLogic.setTicketState(2, 0)
	
		host.sgl_endGame(winner, 3)		

	# update ticket state
	else:
		updateTicketWarning(team, limitId)

	

# called when the ticket state should be updated (for triggering messages and sounds based on tickets left)
def updateTicketWarning(team, limitId):

	oldTicketState = bf2.gameLogic.getTicketState(team)
	newTicketState = 0
	
	if (oldTicketState >= 10):
		newTicketState = 10		

	if (limitId == -2):
		newTicketState = 10
	
	elif (limitId == 2):
		newTicketState = 0		

	elif (limitId == -3):
		newTicketState += 2

	elif (limitId == -4):
		newTicketState += 1

	if (oldTicketState != newTicketState):
		bf2.gameLogic.setTicketState(team, newTicketState)



		

def playerDropFlag(player):

       
        # see if the player was carrying a flag
        global g_flag_playerInx
        global g_timeDroppedFlag

        print "playerDropFlag() player: ", player.getName()

        for i in [1,2]:
           
            if player.index == g_flag_playerInx[i]:
                prnt( str(player.getName()) + " has dropped the flag" )
                resetFlagPos(i)

                if g_allowdroppedflags == 0: #dropped flag implementation
                        g_flag_playerInx[i] = -1
                else:
                        g_flag_playerInx[i] = -2
                        DropTheFlag(player,i)
                        g_timeDroppedFlag[g_flag_otherTeam[i]] = host.timer_getWallTime()

	

def DropTheFlag(player,flagID):
        vehicle = player.getVehicle()
        playerPos = vehicle.getPosition()
        g_PlayerControlObjects = bf2.objectManager.getObjectsOfType('dice.hfe.world.ObjectTemplate.PlayerControlObject')
        for obj in g_PlayerControlObjects:
                if flagID == 1:
                        if obj.templateName == "BlueMiniMapFlag":
                                children = obj.getChildren()
                                obj.setPosition(playerPos)
                                for child in children: 
                                        if child.templateName == "BlueDropFlagCP":
                                                #now set the Red flag trigger and visibility
                                                id = bf2.triggerManager.createHemiSphericalTrigger(child, onCPPickupTrigger, '<<PCO>>', 3, (1, 2, 3))
                                                child.triggerId = id
                                                child.cp_setParam('team',0)
                                                child.cp_setParam('flag',0)
                                                child.cp_setParam('team',2)
                                                child.cp_setParam('flag',2)
                if flagID == 2:
                        if obj.templateName == "RedMiniMapFlag":
                                children = obj.getChildren()
                                obj.setPosition(playerPos)
                                for child in children: 
                                        if child.templateName == "RedDropFlagCP":
                                                #now set the Red flag trigger and visibility
                                                id = bf2.triggerManager.createHemiSphericalTrigger(child, onCPPickupTrigger, '<<PCO>>', 3, (1, 2, 3))
                                                child.triggerId = id
                                                child.cp_setParam('team',0)
                                                child.cp_setParam('flag',0)
                                                child.cp_setParam('team',1)
                                                child.cp_setParam('flag',1)

# called when someone enters a dropped flags radius
def onCPPickupTrigger(triggerId, cp, vehicle, enter, userData):
	global tp
	global g_timeDroppedFlag
	global g_flag_playerInx
	
	if not cp.isValid(): return
	if enter:
                for p in vehicle.getOccupyingPlayers():
                   GetDetails(p)
                   if rootObjectname in g_soldier_types:
                        if (cp.templateName == "BlueDropFlagCP" and  p.getTeam() == 2):
                                host.trig_destroy(triggerId)
                                prnt( "Blue Team Returned the Blue Flag" )
                                g_flag_playerInx[1] = -1
                                HideBlueMiniMapFlag()
                                g_timeDroppedFlag[2]=-1
                        elif (cp.templateName == "RedDropFlagCP" and  p.getTeam() == 1):
                                host.trig_destroy(triggerId)
                                prnt( "Red Team Returned the Red Flag" )
                                g_flag_playerInx[2] = -1
                                HideRedMiniMapFlag()
                                g_timeDroppedFlag[1]=-1
                        elif (cp.templateName == "BlueDropFlagCP" and  p.getTeam() == 1):
                                host.trig_destroy(triggerId)
                                prnt( "Red Team Picked Up the Blue Flag" )
                                g_flag_playerInx[1] = p.index
                                HideBlueMiniMapFlag()
                                g_timeDroppedFlag[2]=-1
                                bf2.gameLogic.sendRankEvent(p, 1, 0) #player has blue flag
                        elif (cp.templateName == "RedDropFlagCP" and  p.getTeam() == 2):
                                host.trig_destroy(triggerId)
                                prnt( "Blue Team Picked Up the Red Flag" )
                                g_flag_playerInx[2] = p.index
                                HideRedMiniMapFlag()
                                g_timeDroppedFlag[1]=-1
                                bf2.gameLogic.sendRankEvent(p, 2, 0) #player has red flag


def resetDroppedFlagonTime(FlagID):
                        if FlagID == 2:
                                prnt( "Blue Flag Returned on Time" )
                                g_flag_playerInx[1] = -1
                                HideBlueMiniMapFlag()
                        elif FlagID == 1:
                                prnt( "Red Flag Returned on Time" )
                                g_flag_playerInx[2] = -1
                                HideRedMiniMapFlag()


def HideBlueMiniMapFlag():
        g_PlayerControlObjects = bf2.objectManager.getObjectsOfType('dice.hfe.world.ObjectTemplate.PlayerControlObject')
        for obj in g_PlayerControlObjects:
                        if obj.templateName == "BlueMiniMapFlag":
                                obj.setPosition((4000.00,4000.00,0.00))
                                children = obj.getChildren()
                                for child in children: 
                                        if child.templateName == "BlueDropFlagCP":
                                                child.cp_setParam('team',0)
                                                child.cp_setParam('flag',0)
def HideRedMiniMapFlag():
        g_PlayerControlObjects = bf2.objectManager.getObjectsOfType('dice.hfe.world.ObjectTemplate.PlayerControlObject')
        for obj in g_PlayerControlObjects:
                        if obj.templateName == "RedMiniMapFlag":
                                obj.setPosition((4000.00,4000.00,0.00))
                                children = obj.getChildren()
                                for child in children: 
                                        if child.templateName == "RedDropFlagCP":
                                                child.cp_setParam('team',0)
                                                child.cp_setParam('flag',0)


def onTimer(data):
        updateFlags()
        checkforsanity()

def checkforsanity():
#check to make sure that only the player with the flag on his team can have a VISIBLE FLAG
        for i in [1,2]: #check team 1 and 2
                if g_flag_playerInx[i] >= 0: #this team has the flag
                        for p in bf2.playerManager.getPlayers():#check eack player
                                if p.getTeam() == i:#on this team
                                        if p.index != g_flag_playerInx[i]:#who does not have the flag
                                                GetDetails(p)
                                                if CarryFlagCP_Object != None:
                                                        CarryFlagCP_Object.cp_setParam('flag',0)#make sure he doesnt have the visible flag showing
                                        elif p.index == g_flag_playerInx[i]:#who does have the flag
                                                GetDetails(p)
                                                if CarryFlagCP_Object != None:
                                                        CarryFlagCP_Object.cp_setParam('flag',g_flag_otherTeam[i])#make sure he does have the visible flag showing
#TODO do we need to add "and CarryFlagCP_Object.isValid():" everywhere?
#and make sure if no one has the flag they do not display the VISIBLE FLAG
                elif g_flag_playerInx[i] < 0: #this team does not have the flag
                        for p in bf2.playerManager.getPlayers():#check eack player
                                if p.getTeam() == i:#on this team
                                                GetDetails(p)
                                                if CarryFlagCP_Object != None and CarryFlagCP_Object.isValid():
                                                        CarryFlagCP_Object.cp_setParam('flag',0)#has no visible flag showing
#now we are going to make sure that the base flags can not have the same teams 
        teamone = -1 
        teamtwo = -2 
        for obj in g_controlPoints: 
            if obj.templateName in g_flag_names: 
                if obj.templateName == g_flag_names[1]: 
                        teamone = obj.cp_getParam('team') 
                if obj.templateName == g_flag_names[2]: 
                        teamtwo = obj.cp_getParam('team') 
        if teamone == teamtwo and teamone != 0: 
                prnt("RESET THE GAME - REPORT THIS TO DEVELOPERS - SHOULD NEVER NEVER HAPPEN!!!!") 
                resetctf()

def resetctf():
        global g_flag_playerInx
        global g_timeDroppedFlag
        global rootObjectname
        g_flag_playerInx = [-1,-1,-1] #reset carried flags 
        g_timeDroppedFlag = [-1,-1,-1] #reset dropped flags 
        rootObjectname = "" #reset global variables 

        for obj in g_controlPoints:
            if obj.templateName in g_flag_names:#this is a base
                if obj.templateName == g_flag_names[1]:#so reset the flags
                        obj.cp_setParam('flag', 1)
                        obj.cp_setParam('team', 1)
                if obj.templateName == g_flag_names[2]:#so reset the flags
                        obj.cp_setParam('flag', 2)
                        obj.cp_setParam('team', 2)

def moveFlag(teamID,Pos):

       # print "moveFlag() named: ", g_carryFlagNames[teamID], " teamID: ", teamID, " pos: ", Pos
        
        g_PlayerControlObjects = bf2.objectManager.getObjectsOfType('dice.hfe.world.ObjectTemplate.PlayerControlObject')

        for obj in g_PlayerControlObjects:
            
            if obj.templateName == g_carryFlagNames[teamID]:

               # print "  flag moved!"
                obj.setPosition(Pos)

def showFlag(teamID,Pos,hideflag): #flag display implementation

                player = bf2.playerManager.getPlayerByIndex(g_flag_playerInx[teamID])
                if player != None:
                        g_PlayerControlObjects = bf2.objectManager.getObjectsOfType('dice.hfe.world.ObjectTemplate.PlayerControlObject')
                        for obj in g_PlayerControlObjects:
                                if hideflag == 0 and obj.templateName == g_miniMapFlagNames[teamID]:
                                        obj.setPosition(Pos)
                        GetDetails(player)
                        if hideflag == 1:
                                CarryFlagCP_Object.cp_setParam('flag',0)
                        else:
                                CarryFlagCP_Object.cp_setParam('flag',0)
                                CarryFlagCP_Object.cp_setParam('flag',g_flag_otherTeam[teamID])

def resetFlagPos(flagID):
                        if g_allowOldStyleFlags == 1: #flag display implementation
                                moveFlag(flagID,(4000.00,4000.00,0.00))
                        else:
                                showFlag(flagID,(4000.00,4000.00,0.00),1)


    
def updateFlags():

        # print "updateFlags()"
        # print " g_flag_playerInx: ", g_flag_playerInx
        i = 0
   
        global g_timeDroppedFlag
        
        for obj in g_controlPoints:

            i += 1
            
            if obj.templateName in g_flag_names:
                if obj.templateName == g_flag_names[1]:
                        team = 1
                if obj.templateName == g_flag_names[2]:
                        team = 2
                otherTeam = g_flag_otherTeam[team]

                # print "  team: ", team, " otherTeam: ", otherTeam

                # Other team does not have this flag
                if g_flag_playerInx[otherTeam] != -1:
                    if obj.cp_getParam('flag') != 0:
                        obj.cp_setParam('flag', 0)
                        obj.cp_setParam('team', 0)

                else:
                    if obj.cp_getParam('flag') != team:
                        obj.cp_setParam('flag', team)
                        obj.cp_setParam('team', team)
                        if team == 1:
                                HideRedMiniMapFlag()
                        if team == 2:
                                HideBlueMiniMapFlag()
        for i in [1,2]:

            if g_timeDroppedFlag[i] != -1 and ((host.timer_getWallTime() - g_timeDroppedFlag[i]) >= g_timeForDroppedFlagsToReturn):
                resetDroppedFlagonTime(i)
                g_timeDroppedFlag[i] = -1

            if g_flag_playerInx[i] == -1:
                
                    # flag is on pole
                    resetFlagPos(i)
                    
            else:
                    # player is carrying the flag
                    
                    # print " updateFlags() # setting pos of flag to playerIndex: ", g_flag_playerInx[i]

                    player = bf2.playerManager.getPlayerByIndex(g_flag_playerInx[i])

                    # print "   player: ", player
                    
                    if player != None:
                        vehicle = player.getVehicle()
                        
                        playerPos = vehicle.getPosition()

                        # print "    playerPos: ", playerPos
                        X = playerPos[0]
                        Y = playerPos[1]+40.0
                        Z = playerPos[2]

                        if g_allowOldStyleFlags == 1: #flag display implementation
                                moveFlag(i,(X,Y,Z))
                        else:
                                showFlag(i,(X,Y,Z),0)
	

			
# called when someone enters or exits cp radius
def onCPTrigger(triggerId, cp, vehicle, enter, userData):
        global playersTeamTickets_end
        global otherTeamTickets_end
        global playersTeam_end
        global otherTeam_end
        global ticketsTeam1
        if not cp.isValid(): return
        if vehicle and vehicle.getParent(): return

        flagName = cp.templateName
        if not flagName in g_flag_names: return

        global g_flag_playerInx
        print "onCPTrigger() flagName: ", flagName, "g_flag_playerInx: ", g_flag_playerInx
        playerID = -1
        playerObj = None
                
        playersInVehicle = None	
        if vehicle:
                playersInVehicle = vehicle.getOccupyingPlayers()
                
                for p in playersInVehicle:
                        GetDetails(p)
                        if not rootObjectname in g_soldier_types: return
                        # only count first player in a vehicle
                        if p == playersInVehicle[0]:  
                                #print "   playerIndex: ", p.index, " is in radius. v=", vehicle.templateName
                                playerObj = p
                                playerID = playerObj.index
                                    
       
        
        if playerID != -1:
            if playerObj != None:
                if playerObj.isAlive() and not playerObj.isManDown():
                    if not playerObj.killed:
                
                        # print "   playerID: ", playerID, " playerTeam: ", playerObj.getTeam(), " cpTeam: ", cp.cp_getParam('team')

                        playersTeam = playerObj.getTeam()
                        
                        if playersTeam == cp.cp_getParam('team'):
                            # This is my team's flag pole
                            
                            # make sure I have the flag
                            if g_flag_playerInx[ playersTeam ] == playerID:

                                # only allow flag capture if other flag is not stolen
                                otherTeam = g_flag_otherTeam[ playersTeam ]
                                if g_flag_playerInx[ otherTeam ] == -1:
                                    #print "  !! CAPTURED THE FLAG !!"
                                    prnt( str(playerObj.getName()) + " has captured the flag for team " + str( g_team_names[playersTeam] ) + "!")
                                    resetFlagPos(playersTeam)
                                    g_flag_playerInx[ playersTeam ] = -1
                                    if playersTeam == 2:
                                        bf2.gameLogic.sendRankEvent(playerObj, 3, 0) #player has capt blue flag
                                    elif playersTeam == 1:
                                        bf2.gameLogic.sendRankEvent(playerObj, 4, 0) #player has capt red flag


                                    print "     Assign Tickets"
                                    teamTickets = bf2.gameLogic.getTickets( playersTeam )
                                    teamTickets += 1
                                    prnt( "CAPTURE LIMIT IS " + str(calcStartTickets(bf2.gameLogic.getDefaultTickets(1))))
                                    bf2.gameLogic.setTickets( playersTeam , teamTickets)
                                    if ((teamTickets - 1000 ) >= calcStartTickets(bf2.gameLogic.getDefaultTickets(1))):
                                        playersTeamTickets = bf2.gameLogic.getTickets( playersTeam )-1000
                                        otherTeamTickets = bf2.gameLogic.getTickets( otherTeam ) -1000
                                        bf2.gameLogic.setTickets( playersTeam , teamTickets)
                                        bf2.gameLogic.setTickets( otherTeam , -1)
                                        playersTeamTickets_end = playersTeamTickets
                                        otherTeamTickets_end = otherTeamTickets
                                        playersTeam_end = playersTeam
                                        otherTeam_end = otherTeam
                                        #bf2.gameLogic.setTickets( playersTeam , playersTeamTickets)
                                        #bf2.gameLogic.setTickets( otherTeam , otherTeamTickets)


                                    # give the player some points
                                    print "     give the player some points"
                                    playerObj.score.cpCaptures += 1
                                    addScore(playerObj, 20, RPL)

                                    # not sure
                                    print "     game event"
                                    bf2.gameLogic.sendGameEvent(playerObj, 12, 0) #12 = Conquest, 0 = Capture


                                
                        else:
                            playersTeam = playerObj.getTeam()
                            # see if flag has not yet been taken and its not my own flag pole and the flag poles team is not set to zero
                            if g_flag_playerInx[ playersTeam ] == -1 and playersTeam != cp.cp_getParam('team') and cp.cp_getParam('team') != 0: 
                                #print "  !! STOLE THE ENEMY FLAG !!"
                                prnt( str(playerObj.getName()) + " has stolen the flag for team " + str( g_team_names[playersTeam] ) + "!")
                                g_flag_playerInx[ playersTeam ] = playerID
                                if playersTeam == 1:
                                        bf2.gameLogic.sendRankEvent(playerObj, 1, 0) #player has blue flag
                                elif playersTeam == 2:
                                        bf2.gameLogic.sendRankEvent(playerObj, 2, 0) #player has red flag
              

        
        # print " g_flag_playerInx: ", g_flag_playerInx
        updateFlags()

			
	
# called when a control point flag reached top or bottom
def onCPStatusChange(cp, top):
        print "onCPStatusChange()"
        

				
				
def onPlayerDeathCQ(victim, vehicle):		
	if not victim: 
		return
	    
	# print "++++++++ onPlayerDeathCQ()"
	
        playerDropFlag(victim)
        updateFlags()


def onPlayerDisconnect(player):
        playerDropFlag(player)
        updateFlags()
	


def onPlayerKilledCQ(victim, attacker, weapon, assists, object):
	if not victim: 
		return

        # print "++++++++ onPlayerKilledCQ()"
        
	victim.killed = True

	playerDropFlag(victim)
        updateFlags()	
                    	        
	
	


			
			
def onPlayerRevived(victim, attacker):

	# update flag takeover status if victim was in a CP radius
	victim.killed = False
	
	cp = getOccupyingCP(victim)
	if cp != None:
		onCPTrigger(-1, cp, victim.getVehicle(), True, None)
	
	
	
def onPlayerSpawn(player, soldier):
        player.killed = False


        


	
	
	
	
def onEnterVehicle(player, vehicle, freeSoldier = False):
	updateCaptureStatus(vehicle)


def onExitVehicle(player, vehicle):

	# update flag takeover status if player in a CP radius
	
	print "Player exiting ", player.getName()
	cp = getOccupyingCP(player)

	# can this cp be captured at all?
	player.setIsInsideCP(cp != None and cp.cp_getParam('unableToChangeTeam') == 0)

	updateCaptureStatus(vehicle)
		

#Update cp capture status on players in vehicle
def updateCaptureStatus(vehicle):	

	rootVehicle = bf2.objectManager.getRootParent(vehicle)
	playersInVehicle = rootVehicle.getOccupyingPlayers()
	
	# set the player in the topmost pco as inside - others outside
	for p in playersInVehicle:
		print "Players in vehicle ", p.getName()
		cp = getOccupyingCP(p)
		p.setIsInsideCP(cp != None and cp.cp_getParam('unableToChangeTeam') == 0 and p == playersInVehicle[0])
		
		
# find cp that player is occupying, if any		
def getOccupyingCP(player):
	vehicle = player.getVehicle()
	playerPos = vehicle.getPosition()
	
	# find closest CP
	closestCP = None
	if len(g_controlPoints) == 0: return None
	for obj in g_controlPoints:
		distanceTo = getVectorDistance(playerPos, obj.getPosition())
		if closestCP == None or distanceTo < closestCPdist:
			closestCP = obj
			closestCPdist = distanceTo
	
	# is the player in radius?
	pcos = bf2.triggerManager.getObjects(closestCP.triggerId)
	for o in pcos:
		if o == player.getDefaultVehicle():
			# Player is DEFAULT vehicle - this is needed when called from onEnterVehicle
			return closestCP
		else:
			for p in o.getOccupyingPlayers():
				if p == player:
					return closestCP
	
	return None



# get distance between two positions
def getVectorDistance(pos1, pos2):
	diffVec = [0.0, 0.0, 0.0]
	diffVec[0] = math.fabs(pos1[0] - pos2[0])
	diffVec[1] = math.fabs(pos1[1] - pos2[1])
	diffVec[2] = math.fabs(pos1[2] - pos2[2])
	 
	return math.sqrt(diffVec[0] * diffVec[0] + diffVec[1] * diffVec[1] + diffVec[2] * diffVec[2])
		
		
	
def FindDetails(physicalObject,level):
# get details for the flags
                global CarryFlagCP_Object
                global rootObjectname

                resultString = physicalObject.templateName

                if resultString == "CarryFlagCP":
                        CarryFlagCP_Object = physicalObject
                if level == 0:
                        rootObjectname=resultString

                kids = physicalObject.getChildren()
                for child in kids:
                        FindDetails(child,level + 1)


def GetDetails(p):
        CarryFlagCP_Object = None
        rootObjectname = ""
        physicalObject = bf2.objectManager.getRootParent(p.getVehicle())
        FindDetails(physicalObject,0)
